@@ -1,10 +1,11 @@
 import json
 import re
 
-import tornado
 from jupyter_server.base.handlers import APIHandler
 from jupyter_server.extension.handler import ExtensionHandlerMixin
 from jupyter_server.utils import ensure_async
+from pydantic import ValidationError
+from tornado.web import HTTPError, authenticated
 
 from jupyter_scheduler.environments import EnvironmentRetrievalError
 from jupyter_scheduler.exceptions import (
@@ -73,112 +74,182 @@ def compute_sort_model(query_argument):
 
 
 class JobDefinitionHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
+    @authenticated
     async def get(self, job_definition_id=None):
         if job_definition_id:
-            job_definition = await ensure_async(
-                self.scheduler.get_job_definition(job_definition_id)
-            )
-            self.finish(job_definition.json())
+            try:
+                job_definition = await ensure_async(
+                    self.scheduler.get_job_definition(job_definition_id)
+                )
+            except SchedulerError as e:
+                self.log.exception(e)
+                raise HTTPError(500, str(e)) from e
+            except Exception as e:
+                self.log.exception(e)
+                raise HTTPError(
+                    500, "Unexpected error occurred while getting job definition details."
+                ) from e
+            else:
+                self.finish(job_definition.json())
         else:
             create_time = self.get_query_argument("create_time", None)
             sort_by = compute_sort_model(self.get_query_arguments("sort_by"))
-            list_query = ListJobDefinitionsQuery(
-                job_definition_id=self.get_query_argument("job_definition_id", None),
-                name=self.get_query_argument("name", None),
-                tags=self.get_query_arguments("tags", None),
-                create_time=int(create_time) if create_time else None,
-                sort_by=sort_by if sort_by else [DEFAULT_SORT],
-                max_items=self.get_query_argument("max_items", DEFAULT_MAX_ITEMS),
-                next_token=self.get_query_argument("next_token", None),
-            )
-            list_response = await ensure_async(self.scheduler.list_job_definitions(list_query))
-            self.finish(list_response.json(exclude_none=True))
+            try:
+                list_query = ListJobDefinitionsQuery(
+                    job_definition_id=self.get_query_argument("job_definition_id", None),
+                    name=self.get_query_argument("name", None),
+                    tags=self.get_query_arguments("tags", None),
+                    create_time=int(create_time) if create_time else None,
+                    sort_by=sort_by if sort_by else [DEFAULT_SORT],
+                    max_items=self.get_query_argument("max_items", DEFAULT_MAX_ITEMS),
+                    next_token=self.get_query_argument("next_token", None),
+                )
+                list_response = await ensure_async(self.scheduler.list_job_definitions(list_query))
+            except ValidationError as e:
+                self.log.exception(e)
+                raise HTTPError(500, str(e)) from e
+            except SchedulerError as e:
+                self.log.exception(e)
+                raise HTTPError(500, str(e)) from e
+            except Exception as e:
+                self.log.exception(e)
+                raise HTTPError(
+                    500, "Unexpected error occurred while getting job definition list."
+                ) from e
+            else:
+                self.finish(list_response.json(exclude_none=True))
 
-    @tornado.web.authenticated
+    @authenticated
     async def post(self):
         payload = self.get_json_body()
         try:
             job_definition_id = await ensure_async(
                 self.scheduler.create_job_definition(CreateJobDefinition(**payload))
             )
+        except ValidationError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
         except InputUriError as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(500, str(e)) from e
+            raise HTTPError(500, str(e)) from e
         except SchedulerError as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(500, str(e)) from e
+            raise HTTPError(500, str(e)) from e
         except Exception as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(
-                500, "Unexpected error occurred during creation of job."
+            raise HTTPError(
+                500, "Unexpected error occurred during creation of job definition."
             ) from e
         else:
             self.finish(json.dumps(dict(job_definition_id=job_definition_id)))
 
-    @tornado.web.authenticated
+    @authenticated
     async def patch(self, job_definition_id):
         payload = self.get_json_body()
-        await ensure_async(
-            self.scheduler.update_job_definition(job_definition_id, UpdateJobDefinition(**payload))
-        )
-        self.set_status(204)
-        self.finish()
+        try:
+            await ensure_async(
+                self.scheduler.update_job_definition(
+                    job_definition_id, UpdateJobDefinition(**payload)
+                )
+            )
+        except ValidationError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except SchedulerError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except Exception as e:
+            self.log.exception(e)
+            raise HTTPError(
+                500, "Unexpected error occurred while updating the job definition."
+            ) from e
+        else:
+            self.set_status(204)
+            self.finish()
 
-    @tornado.web.authenticated
+    @authenticated
     async def delete(self, job_definition_id):
-        await ensure_async(self.scheduler.delete_job_definition(job_definition_id))
-        self.set_status(204)
-        self.finish()
+        try:
+            await ensure_async(self.scheduler.delete_job_definition(job_definition_id))
+        except SchedulerError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except Exception as e:
+            self.log.exception(e)
+            raise HTTPError(
+                500, "Unexpected error occurred while deleting the job definition."
+            ) from e
+        else:
+            self.set_status(204)
+            self.finish()
 
 
 class JobHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
+    @authenticated
     async def get(self, job_id=None):
         if job_id:
-            job = await ensure_async(self.scheduler.get_job(job_id))
-            self.finish(job.json())
+            try:
+                job = await ensure_async(self.scheduler.get_job(job_id))
+            except SchedulerError as e:
+                self.log.exception(e)
+                raise HTTPError(500, str(e)) from e
+            except Exception as e:
+                self.log.exception(e)
+                raise HTTPError(500, "Unexpected error occurred while getting job details.") from e
+            else:
+                self.finish(job.json())
         else:
             status = self.get_query_argument("status", None)
             start_time = self.get_query_argument("start_time", None)
             sort_by = compute_sort_model(self.get_query_arguments("sort_by"))
-            list_jobs_query = ListJobsQuery(
-                job_definition_id=self.get_query_argument("job_definition_id", None),
-                status=Status(status.upper()) if status else None,
-                name=self.get_query_argument("name", None),
-                tags=self.get_query_arguments("tags", None),
-                start_time=int(start_time) if start_time else None,
-                sort_by=sort_by if sort_by else [DEFAULT_SORT],
-                max_items=self.get_query_argument("max_items", DEFAULT_MAX_ITEMS),
-                next_token=self.get_query_argument("next_token", None),
-            )
-            list_jobs_response = await ensure_async(self.scheduler.list_jobs(list_jobs_query))
+            try:
+                list_jobs_query = ListJobsQuery(
+                    job_definition_id=self.get_query_argument("job_definition_id", None),
+                    status=Status(status.upper()) if status else None,
+                    name=self.get_query_argument("name", None),
+                    tags=self.get_query_arguments("tags", None),
+                    start_time=int(start_time) if start_time else None,
+                    sort_by=sort_by if sort_by else [DEFAULT_SORT],
+                    max_items=self.get_query_argument("max_items", DEFAULT_MAX_ITEMS),
+                    next_token=self.get_query_argument("next_token", None),
+                )
+                list_jobs_response = await ensure_async(self.scheduler.list_jobs(list_jobs_query))
+            except ValidationError as e:
+                self.log.exception(e)
+                raise HTTPError(500, str(e)) from e
+            except SchedulerError as e:
+                self.log.exception(e)
+                raise HTTPError(500, str(e)) from e
+            except Exception as e:
+                self.log.exception(e)
+                raise HTTPError(500, "Unexpected error occurred while getting jobs list.") from e
+            else:
+                self.finish(list_jobs_response.json(exclude_none=True))
 
-            self.finish(list_jobs_response.json(exclude_none=True))
-
-    @tornado.web.authenticated
+    @authenticated
     async def post(self):
         payload = self.get_json_body()
         try:
             job_id = await ensure_async(self.scheduler.create_job(CreateJob(**payload)))
+        except ValidationError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
         except InputUriError as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(500, str(e)) from e
+            raise HTTPError(500, str(e)) from e
         except IdempotencyTokenError as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(409, str(e)) from e
+            raise HTTPError(409, str(e)) from e
         except SchedulerError as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(500, str(e)) from e
+            raise HTTPError(500, str(e)) from e
         except Exception as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(
-                500, "Unexpected error occurred during creation of job."
-            ) from e
+            raise HTTPError(500, "Unexpected error occurred during creation of job.") from e
         else:
             self.finish(json.dumps(dict(job_id=job_id)))
 
-    @tornado.web.authenticated
+    @authenticated
     async def patch(self, job_id):
         payload = self.get_json_body()
 
@@ -186,29 +257,46 @@ class JobHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
         status = Status(status) if status else None
 
         if status and status != Status.STOPPED:
-            raise tornado.web.HTTPError(
+            raise HTTPError(
                 500,
                 "Invalid value for field 'status'. Jobs can only be updated to status 'STOPPED' after creation.",
             )
 
-        if status:
-            await ensure_async(self.scheduler.stop_job(job_id))
+        try:
+            if status:
+                await ensure_async(self.scheduler.stop_job(job_id))
+            else:
+                await ensure_async(self.scheduler.update_job(job_id, UpdateJob(**payload)))
+        except ValidationError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except SchedulerError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except Exception as e:
+            self.log.exception(e)
+            raise HTTPError(500, "Unexpected error occurred while updating the job.") from e
         else:
-            await ensure_async(self.scheduler.update_job(job_id, UpdateJob(**payload)))
+            self.set_status(204)
+            self.finish()
 
-        self.set_status(204)
-        self.finish()
-
-    @tornado.web.authenticated
+    @authenticated
     async def delete(self, job_id):
-        await ensure_async(self.scheduler.delete_job(job_id))
-
-        self.set_status(204)
-        self.finish()
+        try:
+            await ensure_async(self.scheduler.delete_job(job_id))
+        except SchedulerError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except Exception as e:
+            self.log.exception(e)
+            raise HTTPError(500, "Unexpected error occurred while deleting the job.") from e
+        else:
+            self.set_status(204)
+            self.finish()
 
 
 class JobFromDefinitionHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
+    @authenticated
     async def post(self, job_definition_id: str):
         payload = self.get_json_body()
         try:
@@ -216,39 +304,54 @@ class JobFromDefinitionHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandl
             job_id = await ensure_async(
                 self.scheduler.create_job_from_definition(job_definition_id, model=model)
             )
+        except ValidationError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
         except SchedulerError as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(500, str(e)) from e
+            raise HTTPError(500, str(e)) from e
         except Exception as e:
             self.log.exception(e)
-            raise tornado.web.HTTPError(
-                500, "Unexpected error occurred during creation of job."
-            ) from e
+            raise HTTPError(500, "Unexpected error occurred during creation of job.") from e
         else:
             self.finish(json.dumps(dict(job_id=job_id)))
 
 
 class BatchJobHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
+    @authenticated
     async def delete(self):
         job_ids = self.get_query_arguments("job_id")
-        for job_id in job_ids:
-            await ensure_async(self.scheduler.delete_job(job_id))
-
-        self.set_status(204)
-        self.finish()
+        try:
+            for job_id in job_ids:
+                await ensure_async(self.scheduler.delete_job(job_id))
+        except SchedulerError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except Exception as e:
+            self.log.exception(e)
+            raise HTTPError(500, "Unexpected error occurred during deletion of jobs.") from e
+        else:
+            self.set_status(204)
+            self.finish()
 
 
 class JobsCountHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
+    @authenticated
     async def get(self):
         status = self.get_query_argument("status", None)
         count_jobs_query = CountJobsQuery(
             status=Status(status.upper()) if status else Status.IN_PROGRESS
         )
-        count = await ensure_async(self.scheduler.count_jobs(count_jobs_query))
-
-        self.finish(json.dumps(dict(count=count)))
+        try:
+            count = await ensure_async(self.scheduler.count_jobs(count_jobs_query))
+        except SchedulerError as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        except Exception as e:
+            self.log.exception(e)
+            raise HTTPError(500, "Unexpected error occurred while getting job count.") from e
+        else:
+            self.finish(json.dumps(dict(count=count)))
 
 
 class RuntimeEnvironmentsHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
@@ -258,7 +361,7 @@ class RuntimeEnvironmentsHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHan
             environments = await ensure_async(self.environments_manager.list_environments())
             output_formats = await ensure_async(self.environments_manager.output_formats_mapping())
         except EnvironmentRetrievalError as e:
-            raise tornado.web.HTTPError(500, str(e))
+            raise HTTPError(500, str(e))
 
         response = []
         for environment in environments:
@@ -271,14 +374,14 @@ class RuntimeEnvironmentsHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHan
 
 
 class FeaturesHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
+    @authenticated
     def get(self):
         cls = self.execution_manager_class
         self.finish(json.dumps(cls.supported_features(cls)))
 
 
 class ConfigHandler(ExtensionHandlerMixin, JobHandlersMixin, APIHandler):
-    @tornado.web.authenticated
+    @authenticated
     def get(self):
         self.finish(
             dict(
@@ -301,10 +404,14 @@ class FilesDownloadHandler(ExtensionHandlerMixin, APIHandler):
 
         return self._job_files_manager
 
-    @tornado.web.authenticated
+    @authenticated
     async def get(self, job_id):
         redownload = self.get_query_argument("redownload", False)
-        await self.job_files_manager.copy_from_staging(job_id=job_id, redownload=redownload)
-
-        self.set_status(204)
-        self.finish()
+        try:
+            await self.job_files_manager.copy_from_staging(job_id=job_id, redownload=redownload)
+        except Exception as e:
+            self.log.exception(e)
+            raise HTTPError(500, str(e)) from e
+        else:
+            self.set_status(204)
+            self.finish()
