@@ -113,10 +113,10 @@ export function AdvancedTable<
     fetchInitialRows();
   }, [props.query]);
 
-  const fetchMoreRows = async () => {
+  const fetchMoreRows = async (newPage: number) => {
     // Do nothing if the next token is undefined (shouldn't happen, but required for type safety)
     if (nextToken === undefined) {
-      return;
+      return false;
     }
 
     // Apply the custom token to the existing query parameters
@@ -131,14 +131,26 @@ export function AdvancedTable<
       })
       .then(payload => {
         setLoading(false);
+        const newRows = props.extractRows(payload) || [];
 
-        // Merge the two lists of jobs and keep the next token from the new response.
+        if (newRows.length === 0) {
+          // no rows in next page -- leave page unchanged, disable next page
+          // button, and show an error banner
+          setNextToken(undefined);
+          setDisplayError(trans.__('Last page reached.'));
+          return;
+        }
+
+        // otherwise, merge the two lists of jobs and keep the next token from
+        // the new response.
         setRows(rows => [
           ...(rows || []),
           ...(props.extractRows(payload) || [])
         ]);
         setNextToken(payload?.next_token);
         setTotalCount(payload?.total_count);
+        setPage(newPage);
+        setMaxPage(newPage);
       })
       .catch((e: Error) => {
         setDisplayError(e.message);
@@ -155,15 +167,15 @@ export function AdvancedTable<
   );
 
   const handlePageChange = async (e: unknown, newPage: number) => {
+    // first clear any display errors
+    setDisplayError(null);
     // if newPage <= maxPage, no need to fetch more rows
     if (newPage <= maxPage) {
       setPage(newPage);
       return;
     }
 
-    await fetchMoreRows();
-    setPage(newPage);
-    setMaxPage(newPage);
+    await fetchMoreRows(newPage);
   };
 
   const onLastPage = page === maxPage && nextToken === undefined;
@@ -181,19 +193,8 @@ export function AdvancedTable<
           // for some reason `to` is set incorrectly on the last page in
           // server-side pagination, so we need to build the string differently
           // in this case.
-
-          // In a case where "from" is larger than the row count, display no message.
-          // This happens when the last page incorrectly has a nextToken.
-          if (from > loadedRows) {
-            return '';
-          }
-
           return trans.__('%1–%2 of %3', from, loadedRows, loadedRows);
         } else {
-          if (from > to) {
-            return '';
-          }
-
           return trans.__(
             '%1–%2 of %3',
             from,
@@ -202,10 +203,6 @@ export function AdvancedTable<
           );
         }
       } else {
-        if (from > to) {
-          return '';
-        }
-
         return trans.__('%1–%2 of %3', from, to, count);
       }
     },
