@@ -1,16 +1,6 @@
 import React, { useMemo, useState } from 'react';
-import {
-  IJobDefinitionModel,
-  JobsView,
-  ICreateJobModel,
-  emptyCreateJobModel
-} from '../../model';
-import { useTranslator } from '../../hooks';
-import { timestampLocalize } from './job-detail';
-import { SchedulerService } from '../../handler';
-import cronstrue from 'cronstrue';
-import { ListJobsTable } from '../list-jobs';
-import { Scheduler as SchedulerTokens } from '../../tokens';
+
+import { JupyterFrontEnd } from '@jupyterlab/application';
 
 import {
   Alert,
@@ -20,46 +10,81 @@ import {
   FormLabel,
   Stack
 } from '@mui/material';
+
+import cronstrue from 'cronstrue';
+
+import { ButtonBar } from '../../components/button-bar';
 import { ConfirmDialogDeleteButton } from '../../components/confirm-dialog-buttons';
-import { JupyterFrontEnd } from '@jupyterlab/application';
 import {
   ILabeledValueProps,
   LabeledValue
 } from '../../components/labeled-value';
+import { SchedulerService } from '../../handler';
+import { useTranslator } from '../../hooks';
+import { ListJobsTable } from '../list-jobs';
+import {
+  IJobDefinitionModel,
+  JobsView,
+  ICreateJobModel,
+  emptyCreateJobModel
+} from '../../model';
+import { Scheduler as SchedulerTokens } from '../../tokens';
+
+import { timestampLocalize } from './job-detail';
 
 export interface IJobDefinitionProps {
   app: JupyterFrontEnd;
-  model: IJobDefinitionModel;
+  model: IJobDefinitionModel | null;
   refresh: () => void;
   setJobsView: (view: JobsView) => void;
   showJobDetail: (jobId: string) => void;
   showCreateJob: (state: ICreateJobModel) => void;
   editJobDefinition: (jobDefinition: IJobDefinitionModel) => void;
   advancedOptions: React.FunctionComponent<SchedulerTokens.IAdvancedOptionsProps>;
+  reload: () => void;
 }
 
 export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
   const trans = useTranslator('jupyterlab');
   const [displayError, setDisplayError] = useState<string | null>(null);
-
   const ss = useMemo(() => new SchedulerService({}), []);
 
+  const ReloadButton = (
+    <Button variant="contained" onClick={props.reload}>
+      {trans.__('Reload Job Definition')}
+    </Button>
+  );
+
+  const ErrorBanner = displayError && (
+    <Alert severity="error">{displayError}</Alert>
+  );
+
+  if (props.model === null) {
+    return (
+      <>
+        {ErrorBanner}
+        <ButtonBar>{ReloadButton}</ButtonBar>
+      </>
+    );
+  }
+  const model: IJobDefinitionModel = props.model;
+
   const handleDeleteJobDefinition = async () => {
-    ss.deleteJobDefinition(props.model.definitionId ?? '')
+    ss.deleteJobDefinition(model.definitionId ?? '')
       .then(_ => props.setJobsView(JobsView.ListJobDefinitions))
       .catch((e: Error) => setDisplayError(e.message));
   };
 
   const pauseJobDefinition = async () => {
     setDisplayError(null);
-    ss.pauseJobDefinition(props.model.definitionId)
+    ss.pauseJobDefinition(model.definitionId)
       .then(_ => props.refresh())
       .catch((e: Error) => setDisplayError(e.message));
   };
 
   const resumeJobDefinition = async () => {
     setDisplayError(null);
-    ss.resumeJobDefinition(props.model.definitionId)
+    ss.resumeJobDefinition(model.definitionId)
       .then(_ => props.refresh())
       .catch((e: Error) => setDisplayError(e.message));
   };
@@ -67,14 +92,15 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
   const runJobDefinition = () => {
     const initialState: ICreateJobModel = {
       ...emptyCreateJobModel(),
-      inputFile: props.model.inputFile,
-      outputPath: props.model.outputPrefix ?? '',
-      environment: props.model.environment,
-      computeType: props.model.computeType,
-      runtimeEnvironmentParameters: props.model.runtimeEnvironmentParameters,
-      parameters: props.model.parameters,
-      outputFormats: props.model.outputFormats,
-      jobDefinitionId: props.model.definitionId
+      jobName: model.name,
+      inputFile: model.inputFile,
+      outputPath: model.outputPrefix ?? '',
+      environment: model.environment,
+      computeType: model.computeType,
+      runtimeEnvironmentParameters: model.runtimeEnvironmentParameters,
+      parameters: model.parameters,
+      outputFormats: model.outputFormats,
+      jobDefinitionId: model.definitionId
     };
 
     props.showCreateJob(initialState);
@@ -83,19 +109,20 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
 
   let cronString;
   try {
-    if (props.model.schedule !== undefined) {
-      cronString = cronstrue.toString(props.model.schedule);
+    if (model.schedule !== undefined) {
+      cronString = cronstrue.toString(model.schedule);
     }
   } catch (e) {
     // Do nothing; let the errors or nothing display instead
   }
 
   const DefinitionButtonBar = (
-    <Stack direction="row" gap={2} justifyContent="flex-end" flexWrap={'wrap'}>
+    <ButtonBar>
+      {ReloadButton}
       <Button variant="outlined" onClick={runJobDefinition}>
         {trans.__('Run Job')}
       </Button>
-      {props.model.active ? (
+      {model.active ? (
         <Button variant="outlined" onClick={pauseJobDefinition}>
           {trans.__('Pause')}
         </Button>
@@ -104,10 +131,7 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
           {trans.__('Resume')}
         </Button>
       )}
-      <Button
-        variant="outlined"
-        onClick={() => props.editJobDefinition(props.model)}
-      >
+      <Button variant="outlined" onClick={() => props.editJobDefinition(model)}>
         {trans.__('Edit Job Definition')}
       </Button>
       <ConfirmDialogDeleteButton
@@ -117,49 +141,49 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
           'Are you sure that you want to delete this job definition?'
         )}
       />
-    </Stack>
+    </ButtonBar>
   );
 
   const jobDefinitionFields: ILabeledValueProps[][] = [
-    [{ value: props.model.name, label: trans.__('Name') }],
+    [{ value: model.name, label: trans.__('Name') }],
     [
       {
-        value: props.model.inputFile,
+        value: model.inputFile,
         label: trans.__('Input file')
       },
       {
-        value: props.model.outputPath,
+        value: model.outputPath,
         label: trans.__('Output directory')
       }
     ],
     [
       {
-        value: props.model.environment,
+        value: model.environment,
         label: trans.__('Environment')
       },
       {
-        value: props.model.active ? trans.__('Active') : trans.__('Paused'),
+        value: model.active ? trans.__('Active') : trans.__('Paused'),
         label: trans.__('Status')
       }
     ],
     [
       {
-        value: timestampLocalize(props.model.createTime ?? ''),
+        value: timestampLocalize(model.createTime ?? ''),
         label: trans.__('Created at')
       },
       {
-        value: timestampLocalize(props.model.updateTime ?? ''),
+        value: timestampLocalize(model.updateTime ?? ''),
         label: trans.__('Updated at')
       }
     ],
     [
       {
-        value: props.model.schedule ?? '',
+        value: model.schedule ?? '',
         helperText: cronString ?? '',
         label: trans.__('Schedule')
       },
       {
-        value: props.model.timezone ?? '',
+        value: model.timezone ?? '',
         label: trans.__('Time zone')
       }
     ]
@@ -195,7 +219,7 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
           </FormLabel>
           <props.advancedOptions
             jobsView={JobsView.JobDefinitionDetail}
-            model={props.model}
+            model={model}
             handleModelChange={(_: any) => {
               return;
             }}
@@ -217,7 +241,7 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
             app={props.app}
             showCreateJob={props.showCreateJob}
             showJobDetail={props.showJobDetail}
-            jobDefinitionId={props.model.definitionId}
+            jobDefinitionId={model.definitionId}
             pageSize={5}
             emptyRowMessage={trans.__(
               'No notebook jobs associated with this job definition.'
@@ -230,7 +254,7 @@ export function JobDefinition(props: IJobDefinitionProps): JSX.Element {
 
   return (
     <>
-      {displayError && <Alert severity="error">{displayError}</Alert>}
+      {ErrorBanner}
       {DefinitionButtonBar}
       {JobDefinition}
       {JobsList}
