@@ -129,6 +129,8 @@ export interface IUpdateJobDefinitionModel
   definitionId: string;
   name: string;
   environment: string;
+  inputFileSnapshot: string;
+  updateTime: number;
 }
 
 export function emptyUpdateJobDefinitionModel(): IUpdateJobDefinitionModel {
@@ -136,6 +138,8 @@ export function emptyUpdateJobDefinitionModel(): IUpdateJobDefinitionModel {
     definitionId: '',
     name: '',
     environment: '',
+    inputFileSnapshot: '',
+    updateTime: 0,
     ...defaultScheduleFields
   };
 }
@@ -328,7 +332,7 @@ export interface IJobDefinitionModel {
   definitionId: string;
   active?: boolean;
   createTime?: number;
-  updateTime?: number;
+  updateTime: number;
   startTime?: number;
   endTime?: number;
   outputPrefix?: string;
